function[RandSynthData] = SyntheticDOSY(SynthData)

% Generate a DOSY experiment
sw=SynthData.sw;
np=SynthData.np;
mingrad=SynthData.mingrad;
maxgrad=SynthData.maxgrad;
ngrad=SynthData.ni;
npeaks=SynthData.npeaks;
noise=SynthData.noise;
gw=SynthData.gw';
lw=SynthData.lw';
NUGs=SynthData.NUGs;
Amps=SynthData.Amps;

at=np/sw;
dwell=1/sw;
t=0:dwell:(at-dwell);
% hz=0:(2500/(np-1)):(sw);
% hz=hz-1250;

% Useful Arrays
Timescale=linspace(0,at,np);
nparray=linspace(1,32768,np);

% Add lineshape (Gaussian)
for g=1:npeaks
    if gw(g)~=0
        gf(g)=2*sqrt(log(2))/(pi*gw(g));
        GwFunc(g,:)=exp(-(Timescale'/gf(g)).^2);
    else
        GwFunc(g,:)=ones(1,np);
    end
end

% Add lineshape (Lorentzian)
for l=1:npeaks
    if lw~=0
        LbFunc(l,:)=exp(-Timescale'*pi*lw(l));
    else
        LbFunc(l,:)=ones(1,np);
    end
end

% Combine the window functions.
Winfunc=GwFunc.*LbFunc;

if isfield(SynthData,'Freqs')
% Use the values from the Table in the Toolbox
    omega=(SynthData.Freqs');
else
% Generate a number(npeaks) of random frequencies within 1/4 to 3/4 sw.
    omega=(0.5*sw*(rand(npeaks,1)-1.5)+sw/2);
end

% Frequencies into rotating decaying exponentials
Signals={npeaks};
    for k=1:npeaks
        Signals{k}=Amps(k)*exp(-1i*2*pi*omega(k)*t);
        Signals{k}=Signals{k}.*Winfunc(k,:);
        Signals{k}=repmat(Signals{k},ngrad,1);
    end

% Generate some DOSY decays
    % First define some parameters
    gamma=2.67522212e8;
    Dmax=20;
    del=0.004;
    DEL=0.1;
    dosyconstant=gamma^2*del^2*DEL;
    
    % calculate the gradient array
    gradlvl=zeros(1,ngrad);
    for p=1:ngrad
        gradlvl(p)=mingrad^2+(p-1)*(maxgrad^2-mingrad^2)/(ngrad-1); 
    end
    gradlvl=sqrt(gradlvl);

if isfield(SynthData,'Dvals')
% Use the values from the Table in the Toolbox    
    Dvals=SynthData.Dvals*1e-10';
else
% generate some random D
%     Dvals=(Dmax.*rand(1,npeaks))*1e-10;
    dtemp=round((Dmax.*rand(1,npeaks))*1000)/1000;
    Dvals=dtemp*1e-10;
end
    
    % form a decay matrix
    decays={npeaks};
    for p=1:npeaks
    expfactor=Dvals(p)*gamma^2*del^2*DEL.*gradlvl.^2;
        if NUGs==0      
            decays{p}=exp(-expfactor);
            decays{p}=repmat(decays{p},np,1)';
        else
            decays{p} = exp( - NUGs(1)*expfactor.^1 -...
            NUGs(2)*expfactor.^2 -...
            NUGs(3)*expfactor.^3 -...
            NUGs(4)*expfactor.^4);
            decays{p}=repmat(decays{p},np,1)';
        end   
    end
    % Apply the decays
    DOSYcell={npeaks};
    for f=1:npeaks
        DOSYcell{f}=Signals{f}.*decays{f};
    end
    % Merge Cell matrix into a DOSY dataset
    DOSYdata=zeros(ngrad,np);
    for h=1:npeaks
        DOSYdata=DOSYdata+DOSYcell{h};
    end
    
DOSYFIDs=DOSYdata;
% to add the right amount of noise: fft, add noise, ifft.
DOSYspect=fft(DOSYFIDs');

if SynthData.noise==0
    GaussianNoise=0;
else
% Add some noise
GaussianNoise=((randn(ngrad,np)+1i*randn(ngrad,np)).*max(max(real(DOSYspect))))...
    ./SynthData.noise;
end

DOSYspect=DOSYspect+GaussianNoise';

% 3D plot
if SynthData.meshplot==1
    figure
    mesh(real(fftshift(DOSYspect,1)))       
end


for f=1:ngrad
DOSYFIDs(f,:)=ifft(DOSYspect(:,f));
end

TimeErrFlag=0;
if TimeErrFlag==1
    DOSYFIDs=DOSYFIDs(:,3:end);
end


% Prepare data for exporting
RandSynthData.Freqs=omega;
RandSynthData.SPECTRA=DOSYspect';
RandSynthData.FIDs=DOSYFIDs';
RandSynthData.npeaks=npeaks;
RandSynthData.sw=sw;
RandSynthData.np=np;
RandSynthData.ngrad=ngrad;
RandSynthData.maxgrad=maxgrad;
RandSynthData.mingrad=mingrad;
RandSynthData.noise=noise;
RandSynthData.decays=decays;
RandSynthData.Dvals=Dvals;
RandSynthData.delta=del;
RandSynthData.DELTA=DEL;
RandSynthData.at=at;
RandSynthData.gradlvl=gradlvl;
RandSynthData.gamma=gamma;
RandSynthData.dosyconstant=dosyconstant;
RandSynthData.Amps=Amps;
RandSynthData.Lw=lw;
RandSynthData.Gw=gw;
end