function [dosydata]=fdmrrt_mn(NmrData)


%   [DOSY]=dosy_mn(pfgnmrdata,thresh,Specrange,Diffrange,Options,Nugc)   
%   DOSY (diffusion-ordered spectroscopy) fitting of
%   PFG-NMR diffusion data (aka DOSY data)
%   
%   -------------------------------INPUT--------------------------------------
%   pfgnmrdata      Data structure containing the PFGNMR experiment.
%                   containing the following members:
%                       filename: the name and path of the original file
%                       np: number of (real) data points in each spectrum
%                       wp: width of spectrum (ppm)
%                       sp: start of spectrum (ppm)
%                       dosyconstant: gamma.^2*delts^2*DELTAprime
%                       Gzlvl: vector of gradient amplitudes (T/m)
%                       ngrad: number of gradient levels
%                       Ppmscale: scale for plotting the spectrum
%                       SPECTRA: spectra (processed)
%   ---------------------------INPUT - OPTIONAL-------------------------------
%   thresh =        Noise threshold. Any data point below won't be used in
%                   the fitting. Expressed as % of highest peak. Default 5
%                   (%).
%   Specrange       The spectral range (in ppm) in which the score fitting
%                   will be performed. if set to [0] defaults will be used.
%                   DEFAULT is [sp wp+sp];
%   Diffrange =     The range in diffusion coeeficients that will be
%                   calculated AND the number of data points in the
%                   diffusion dimension (fn1). DEFAULT is [0 20 128];
%                     Diffrange(1) - Min diffusion coefficient plotted.
%                              (0)   DEFAULT. (*10e-10 m2/s)
%                     Diffrange(2) - Max diffusion coefficient plotted.
%                              (20)  DEFAULT  (*10e-10 m2/s)
%                     Diffrange(3) - Number of datapoints in the diffusion
%                                    dimension
%                              (128) DEFAULT
%
%   Options =       Optional parameters. If not given or set to zero or [],
%                   defaults will be used. If you want Options(5) to be 2
%                   and not change others, simply write Options(5)=2. Even
%                   if Options hasn't been defined Options will contain
%                   zeros except its fifth element.
%
%           Options(1) - peak picking.
%             (0) DEFAULT does peak picking;
%             (1) fits each frequency individually
%
%           Options(2) - fitting functions.
%             (0)  DEFAULT: fits to a pure monoexponential.
%             (1)  fits to a function compensating for non-uniform
%                  gradients (NUG); Supply your own coefficients or use the
%                  default (Varian ID 5mm probe, Manchester 2006).Max 4
%                  coefficients is supported.
%
%           Options(3) - Multiexponential fiting. The max number of
%                        componets per peak/data point. Warning - the higer
%                        number the longer the calculation. 2 is often the
%                        practical limit.
%             (1)  DEFAULT: - monoexponental fit
%
%           Options(4) - The number of random starting values (Monte
%                        Carlo) that will be tried for the biexponetial
%                        fit. Default is 100. Higher exponentials will be
%                        corrspondingly increased (i.e 1000 as a default.
%                        If you get a successful fit with a low number of
%                        random starting values it probably means that the
%                        data is multiexponetial but you may not have found
%                        the globl minimum. It may be worthwhile to
%                        increase it to increase the likelyhood of getting
%                        a correct result.
%           Options(5) - The fitting routine used
%               (0) DEFAULT: - lsqcurvefit. Required the Otimization Toolbox
%                   and uses gradients. Probably the fastest and most reliable
%                   option.
%               (1) fminsearch: uses the Nelder-Mead simplex method. 
%
%   Nugc =          Vector contaning coefficients for non-uniform field
%                   gradient correction. If not supplied default values
%                   from a Varian ID 5mm probe (Manchester 2006) will be
%                   used.U0_R2C(:,j2+1)
%
%   --------------------------------------OUTPUT------------------------------
%   dosydata       Structure containg the data obtained after dosy with
%                  the follwing elements:
%                     
%                  
%                  fit_time:   The time it took to run the fitting
%                  Gzlvl: vector of gradient amplitudes (T/m)
%                  wp: width of spectrum (ppm)
%                  sp: start of spectrum (ppm)
%                  Ppmscale: scale for plotting the spectrum
%                  filename: the name and path of the original file
%                  Options: options used for the fitting
%                  Nug: coeffients for the non-uniform field gradient
%                       compensation
%                  FITSTATS: amplitude and diffusion coeffcient with 
%                            standard error for the fitted peaks
%                  FITTED: the fitted decys for each peak
%                  freqs: frequencies of the fitted peaks (ppm)
%                  RESIDUAL: difference between the fitted decay and the
%                            original (experimental) for all peaks
%                  ORIGINAL: experimental deacy of all fitted peaks
%                  type: type of data (i.e. dosy or something else)
%                  DOSY: matrix containing the DOSY plot
%                  Spectrum: the least attenuated spectrum      
%                  fn1: number of data points (digitisation) in the
%                       diffusion dimension
%                  dmin: plot limit in the diffusion dimension 10-10 m^2/s)
%                  dmax: plot limit in the diffusion dimension 10-10 m^2/s)
%                  Dscale: Plot scale for the diffusion dimension       
%                  threshold: theshold (%) over which peaks are included in
%                             the fit
%                   
%   Example:  
%
%   See also: dosy_mn, score_mn, decra_mn, mcr_mn, varianimport, 
%             brukerimport, jeolimport, peakpick_mn, dosyplot_mn, 
%             dosyresidual, dosyplot_gui, scoreplot_mn, decraplot_mn,
%             mcrplot_mn
%             
%   References:
%   Nilsson M, Connell, MA, Davis, AL, Morris, GA Anal. Chem.
%   2006; 78: 3040.

%   
%   
%   This is a part of the DOSYToolbox        
%   Copyright  2007-2008  <Mathias Nilsson>%
%   This program is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or F1ITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for mor1e details.
%
%   You should have received a copy of the GNU General Public License along
%   with this program; if not, write to the Free Software Foundation, Inc.,
%   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
%
%   Dr. Mathias Nilsson
%   School of Chemistry, University of Manchester,
%   Oxford Road, Manchester M13 9PL, UK
%   Telephone: +44 (0) 161 306 4465
%   Fax: +44 (0)161 275 4598
%   mathias.nilsson@manchester.ac.uk

if nargin==0
    disp(' ')
    disp(' FDMRRT')
    disp(' ')
    disp(' Type <<help fdmrrt_mn>> for more info')
    disp('  ')
    return
elseif nargin<1
    error(' dosy_mn needs a NmrData stucture as input')
elseif nargin > 1
    error(' Too many inputs')
end

t_start=cputime;

%Defaults
pfgnmrdata = NmrData.pfgnmrdata;
thresh = NmrData.th;
sfrq = NmrData.sfrq;
fid = NmrData.FID;
SPECTRA = NmrData.SPECTRA;
opts = NmrData.FDMRRTopts;

% User options
dscale=1e-10;

%%%
format('longG')

SOURCE_FID = 0;
SOURCE_SPEC = 1;
ALGO_FDM = 0;
ALGO_RRT = 1;
SPEC_ABS = 0;
SPEC_PSEUDO = 1;

source = opts(1);
algorithm = opts(2);
spec_type = opts(3);
N1 = opts(4);
N2 = opts(5);
Kwin = opts(6);
q = opts(7);
w2min = opts(8) * dscale;
w2max = opts(9) * dscale;
nspec2 = opts(10);
nspec1 = opts(11);
lrfid = opts(20);

M1 = N1/2 - 1;
M2 = N2/2 - 1;
rho = 1.1;
tau1 = 2*pi / pfgnmrdata.wp / sfrq;
tau2 = pfgnmrdata.dosyconstant * ...
	(pfgnmrdata.Gzlvl(pfgnmrdata.ngrad)^2-pfgnmrdata.Gzlvl(1)^2) / ...
	(pfgnmrdata.ngrad-1);
%w1min = -pfgnmrdata.wp/2 * sfrq
%w1max = pfgnmrdata.wp/2 * sfrq
w1min = (NmrData.xlim_spec(1) - pfgnmrdata.wp/2 - pfgnmrdata.sp)*sfrq;
w1max = (NmrData.xlim_spec(2) - pfgnmrdata.wp/2 - pfgnmrdata.sp)*sfrq;
wstep = 2*pi / (tau1*(M1+1)*rho);
window_size = Kwin * wstep;
spec1step = (w1max-w1min) / nspec1;
spec2step = (w2max-w2min) / nspec2;
g1 = 1/(5000*tau1);
g2 = 1/(5000*tau2);
e_tau1g = exp(-tau1*g1);
e_itau2g = exp(i*tau2*g2);

if(source == SOURCE_FID) 
	fid = circshift(fid,-lrfid);
	fid(end-lrfid+1:end,:) = 0.0;
else
	fid=ifft(flipud(fftshift(SPECTRA,1)));
	fid(1,:)=fid(1,:)./NmrData.fpmult;
end

single_window = 0;
if (single_window | window_size > 2 * (w1max-w1min))
	windows = 1;
	single_window = 1;
	Kspec = Kwin;
else
	windows = ceil(2*(w1max-w1min)/window_size+1);
	Kspec = Kwin * (windows+1)/2.0;
end

z_ = zeros(Kspec,1);
z_m = zeros(Kspec,1);
for j=1:Kspec
	if(single_window)
		wj = (w1min + w1max - window_size)/2.0 + j*wstep;
	else
		wj = w1min - window_size/2.0 + j*wstep;
	end
	z_(j) = exp(i*tau1*wj); % 1/z
	z_m(j) = z_(j)^(M1);
end

ap = zeros(N1-1,3);
for n2=0:N2-2
	triangle = (M2 - abs(M2-n2) + 1);
	for n1=0:N1-2
			ap(n1+1,0+1) = ap(n1+1,0+1) + fid(n1+0+1,n2+0+1) * triangle;
			ap(n1+1,1+1) = ap(n1+1,1+1) + fid(n1+1+1,n2+0+1) * triangle;
			ap(n1+1,2+1) = ap(n1+1,2+1) + fid(n1+0+1,n2+1+1) * triangle;
	end
end

%cm = zeros(Kspec,N2);
%for n2=0:N2-2
%	cm(:,n2+1) = Slow_FT(M1, fid(1+1:M1+1,n2+1), Kspec, z_);
%	cm(:,n2+1) = cm(:,n2+1) + fid(1,n2+1);
%end
%c_sum = sum(cm(:,1:M2+1),2);
c_sum = Slow_FT(M1, sum(fid(2:M1+1,1:M2+1),2), Kspec, z_);
c_sum = c_sum + sum(fid(1,1:M2+1),2);

%c_fn = zeros(M1,1);
sum_f = zeros(Kspec,3);
sum_g = zeros(Kspec,3);
sum_h = zeros(Kspec,3);

hp = waitbar(0,'common fourier elements');
for p=0:2
%	for n1=1:M1
%		c_fn(n1) = (n1+1) * ap(n1+1,p+1);
%	end
	c_fn = (2:M1+1)' .* ap(2:M1+1,p+1);
	sum_h(:,p+1) = Slow_FT(M1, c_fn(1:M1), Kspec, z_);
	waitbar((4*p+1)/(3*4),hp);
%	for n1=1:M1
%		c_fn(n1) = (n1+1) * ap(n1+M1+1,p+1);
%	end
	c_fn = (2:M1+1)' .* ap(M1+2:2*M1+1,p+1);
	sum_f(:,p+1) = Slow_FT(M1, c_fn(1:M1), Kspec, z_);
	waitbar((4*p+2)/(3*4),hp);
	sum_g(:,p+1) = Slow_FT(M1, ap(M1+1+1:2*M1+1,p+1), Kspec, z_);
	waitbar((4*p+3)/(3*4),hp);
%	for j=1:Kspec
%		sum_g(j,p+1) = sum_g(j,p+1) * z_m(j);
%		sum_h(j,p+1) = sum_h(j,p+1) + ap(0+1,p+1) + (M1+2) * ...
%			sum_g(j,p+1) - sum_f(j,p+1) * z_m(j);
%	end
	sum_g(:,p+1) = sum_g(:,p+1) .* z_m(:);
	sum_h(:,p+1) = sum_h(:,p+1) + ap(0+1,p+1) + (M1+2) * ...
		sum_g(:,p+1) - sum_f(:,p+1) .* z_m(:);
	sum_f(:,p+1) = Slow_FT(M1, ap(1+1:M1+1,p+1), Kspec, z_);
	waitbar((4*p+4)/(3*4),hp);
end

matrix_scale = 0.0;
for j=1:Kspec
	matrix_scale = matrix_scale + abs(sum_h(j,0+1));
end
matrix_scale = matrix_scale / Kspec;

S_2d = zeros(nspec1, nspec2);

uk = zeros(Kwin,windows);
wk = zeros(Kwin,windows);
ak = zeros(Kwin,windows);
dk = zeros(Kwin,windows);
window_limit = zeros(windows,2);

waitbar(0,hp,['rendering window ', num2str(0),' of ', num2str(windows)])
for window=1:windows
	if(single_window)
		lwindow = 0.5*(w1min+w1max-window_size);
	else
		lwindow = w1min + 0.5*(window-2)*window_size;
	end
	rwindow = lwindow + window_size;
	window_limit(window,:) = [lwindow, rwindow];
	basis_offset = floor(0.5*(window-1)*Kwin);
	Up = zeros(Kwin, Kwin, 3);
	for p=0:2
		for j=1:Kwin
			z_2m = z_m(basis_offset+j)^2;
			Up(j,j,p+1) = sum_h(basis_offset+j,p+1);
%			disp(Up(j,j,p+1))
        end
		for j1=1:Kwin
			for j2=1:j1-1
				% off-diagonal terms (upper triangle)
				z_z = z_(basis_offset+j1) / z_(basis_offset+j2);
				Up(j1,j2,p+1) = ap(0+1,p+1) + (sum_f(basis_offset+j2,p+1) - ...
					z_z^(M1+1) * sum_g(basis_offset+j2,p+1) - ...
					z_z * sum_f(basis_offset+j1,p+1) + ...
					z_z ^(-M1) * sum_g(basis_offset+j1,p+1)) / (1-z_z);
					% mirror upper triangle onto lower triangle
					Up(j2,j1,p+1) = Up(j1,j2,p+1);
			end
		end
	end
	
	spec1min_win = floor(max((lwindow-w1min)*nspec1 / (w1max-w1min),0.0));
	spec1max_win = floor(min((rwindow-w1min)*nspec1 / (w1max-w1min),nspec1+0.0))-1;

	if(algorithm == ALGO_FDM)
		[U,S,V] = svd(Up(:,:,0+1));
		for j1=1:Kwin
			if(q*matrix_scale > S(j1,j1));
				S(j1,j1) = q*matrix_scale;
			end
		end
		Up(:,:,0+1) = U * S * V';
		U0 = Up(:,:,0+1);
		[b1, u1] = eig(Up(:,:,1+1),Up(:,:,0+1),'qz');
%		Up(:,:,1+1)* b1(:,k1) - u1(k1) * Up(:,:,0+1) * b1(:,k1)
		u1 = diag(u1);
		w = -log(u1)/(i * tau1);
%		b1(:,k1) = b1(:,k1) / sqrt(conj(b1(:,k1)') * U0 * b1(:,k1));
%		d1 = (conj(b1(:,k1)') * c_sum(basis_offset+1:basis_offset+Kwin))^2
		b1 = b1 / sqrt(conj(b1') * U0 * b1);
		d1 = conj(b1') * c_sum(basis_offset+1:basis_offset+Kwin);
%		d1 = d1 .* d1;
		uk(:,window) = u1;
		wk(:,window) = real(w);
		dk(:,window) = d1;
		for k1=1:Kwin
			if(imag(w(k1)) > 0)
					u1(k1) = (u1(k1) / abs(u1(k1))^2);
			end
			if(abs(imag(w(k1))) < g1)
				u1(k1) = exp(-i * tau1 * real(w(k1))) * e_tau1g;
			end
		end
		[b2, u2] = eig(Up(:,:,2+1),Up(:,:,0+1),'qz');
		u2 = diag(u2);
		alph = -log(u2)/tau2;
		ak(:,window) = alph;
		b2 = b2 / sqrt(conj(b2') * U0 * b2);
		d2 = conj(b2') * c_sum(basis_offset+1:basis_offset+Kwin);
		d2 = d2 .* d2;
		for k2=1:Kwin
			if(abs(imag(alph(k2))) < g2)
				u2(k2) = exp(-tau2 * real(alph(k2))) * e_itau2g;
			end
		end
%		DD1(k1,k2) = conj(b1(:,k1)') * U0 * b2(:,k2);
		D = conj(b1') * U0 * b2;
	end

	right = zeros(Kwin,nspec2);
	for j2=0:nspec2-1
		aj = w2min + j2*spec2step;
		lj = exp(-tau2*aj);
		if(algorithm == ALGO_FDM)
			if(spec_type == SPEC_PSEUDO)
				right(:,j2+1) = tau2^2 * D * ...
					((1 - (lj * ones(Kwin, 1) ./ u2)) .^(-2) .* d2);
			else
				right(:,j2+1) = tau2 * D * ...
					((1 - (lj * ones(Kwin, 1) ./ u2)) .^(-1) .* d2);
			end
		else
			R = Up(:,:,0+1) - Up(:,:,2+1) / lj;
			[U,S,V] = svd(R);
			for k1=1:Kwin
				for k2=1:Kwin
					VS(k1,k2) = V(k1,k2) * S(k2,k2) / (S(k2,k2)^2 + q * matrix_scale);
				end
			end
			R_ = VS * U';
			U0_R2 = Up(:,:,0+1) * R_;
			right(:,j2+1) = U0_R2 * c_sum(basis_offset+1:basis_offset+Kwin);
			if(spec_type == SPEC_PSEUDO)
				right(:,j2+1) = U0_R2 * right(:,j2+1);
			end
		end
	end

	for j1=spec1min_win:spec1max_win
		wj = w1min + j1*spec1step;
		zj = exp(-i*tau1*wj);
		if(single_window)
			weight = 1.0;
		else
			weight = cos((wj-lwindow)*2*pi/(rwindow-lwindow)+pi)*0.5 + 0.5;
		end
		if(algorithm == ALGO_FDM)
			if(spec_type == SPEC_PSEUDO)
				left = tau1^2 * (1 - (zj*ones(Kwin,1) ./ u1)).^(-2) .* d1;
			else
				left = tau1 * (1 - (zj*ones(Kwin,1) ./ u1)).^(-1) .* d1;
			end
		else
			R = Up(:,:,0+1) - Up(:,:,1+1) / zj;
			[U,S,V] = svd(R);
			for k1=1:Kwin
				for k2=1:Kwin
					VS(k1,k2) = V(k1,k2) * S(k2,k2) / (S(k2,k2)^2 + q * ...
						matrix_scale);
				end
			end
			R_ = VS * U';
			left = transpose(c_sum(basis_offset+1:basis_offset+Kwin)) * R_;
			if(spec_type == SPEC_PSEUDO)
				left = left *  Up(:,:,0+1) * R_;
			end
		end
		for j2=0:nspec2-1
			S_2d(j1+1,j2+1) = S_2d(j1+1,j2+1) + weight * sum(right(:,j2+1) .* left(:));
		end
	end
	waitbar(window/windows,hp,['rendering window ', num2str(window), ...
		' of ',  num2str(windows)]);
end
close(hp);

%expfactor=pfgnmrdata.dosyconstant*dscale;
%pfgnmrdata.Gzlvl=pfgnmrdata.Gzlvl.^2;
%[nspec,ngrad]=size(pfgnmrdata.SPECTRA);

th = thresh/100;
if(th <= 0)
	th = 0.10;
end

DOSY = abs(S_2d);
norm = DOSY - min(DOSY(:));
DOSY = norm / max(norm(:));
DOSYth = min(DOSY,th);

endt=cputime-t_start;
h=fix(endt/3600);
m=fix((endt-3600*h)/60);
s=endt-3600*h - 60*m;
fit_time=[h m s];
disp(['DOSY: Fitting time was: ' num2str(fit_time(1)) ' h  ' num2str(fit_time(2)) ' m and ' num2str(fit_time(3),2) ' s']);

spec_samples = floor(linspace(1, size(pfgnmrdata.SPECTRA,1), nspec1));

dosydata.type='fdm/rrt';
dosydata.fit_time=fit_time;
dosydata.DOSY=DOSY;
dosydata.Gzlvl=pfgnmrdata.Gzlvl;

%dosydata.Ppmscale=pfgnmrdata.Ppmscale';
dosydata.Ppmscale=linspace(w1min,w1max,nspec1)/sfrq - (-pfgnmrdata.wp/2-pfgnmrdata.sp);
%dosydata.Spectrum=imresize(pfgnmrdata.SPECTRA(:,1),[nspec1,1]);
dosydata.Spectrum = pfgnmrdata.SPECTRA(floor(...
	linspace(1,size(pfgnmrdata.SPECTRA,1),nspec1)));
dosydata.sp=pfgnmrdata.sp;
dosydata.wp=pfgnmrdata.wp;
dosydata.np=pfgnmrdata.np;
dosydata.fn1=nspec2;
dosydata.dmin=w2min;
dosydata.dmax=w2max;
dosydata.Dscale=linspace(w2min,w2max,nspec2);
dosydata.Options=opts;
dosydata.threshold=th;
dosydata.uk = uk;
dosydata.wk = -(wk/sfrq - (-pfgnmrdata.wp/2-pfgnmrdata.sp));
dosydata.dk = dk.^2;
dosydata.ak = ak;
dosydata.windows = window_limit;
%-(window_limit/sfrq - (-pfgnmrdata.wp/2-pfgnmrdata.sp));
%dosydata
%close(hp);

	dosyfigure = figure(...
		'Name','Dosy Plot',...
		'NumberTitle','Off');
	set(zoom(dosyfigure), 'ActionPostCallback', @zoomer);
	set(rotate3d(dosyfigure), 'ButtonDownFilter', @fixer);
	dosyaxes = axes('Position',[0.05, 0.2, 0.9, 0.75]);
	[xax,yax]=meshgrid(dosydata.Ppmscale,dosydata.Dscale);
	dosy_plot = contour(-fliplr(xax),-flipud(yax), fliplr(flipud(DOSYth')));
	xlimits = get(dosyaxes, 'XLim');
	ylimits = get(dosyaxes, 'YLim');
	zmax = max(DOSYth(:));
	windowpane = patch([xlimits(1),xlimits(2),xlimits(2),xlimits(1)],...
		[ylimits(2),ylimits(2),ylimits(1),ylimits(1)], ...
		[zmax,zmax,zmax,zmax], ...
		'Blue','FaceAlpha',0.0,'LineStyle','none','ButtonDownFcn',@click_callback);
	dfdata = struct('wmin', w1min, 'wmax', w1max, ...
		'wmin_ppm', xlimits(1), 'wmax_ppm', xlimits(2), ...
		'nclicks', 0, 'last_y', 0, 'last_line', 0, 'Regions', [], ...
		'window', windowpane, 'windows', dosydata.windows, ...
		'wk', dosydata.wk, 'dk', dosydata.dk, 'ak', dosydata.ak, ...
		'uk', dosydata.uk, 'tau1', tau1, 'wp', dosydata.wp, 'sp', dosydata.sp, ...
		'sfrq', sfrq);
	guidata(dosyfigure, dfdata);
	uirender = uicontrol(dosyfigure,'Style','PushButton','String','Render', ...
		'Units', 'Normalized','Position',[0.05, 0.03, 0.9, 0.1], ...
		'Callback', @CallbackRender);

%dosyplot_gui(dosydata,5,th);
end

%-----------------------END of fdmrrt_mn-------------------------------------


%-----------------Auxillary functions--------------------------------------
function [f] = Slow_FT(M,coef,Nz,z)
	f = zeros(Nz,1);
	for j=1:Nz
		f(j) = 0.0;
%		disp(coef(M))
%		disp(z(j))
%		disp(coef(M)*z(j))
		fff = coef(M)*z(j);
		if(M == 1)
			f(j) = fff;
			continue;
		end
		for n=M-1:-1:1
			fff = (fff+coef(n))*z(j);
			if(mod(n,100) == 0 | n == 1)
				f(j)=f(j)+fff*z(j)^(n-1);
	      fff=0.0;
			end
		end
	end
end

function click_callback(source, eventdata)
	dosyfigure = gcf;
	dosyaxes = get(dosyfigure,'CurrentAxes');
	dfdata = guidata(dosyfigure);
	button = get(dosyfigure,'SelectionType');
	pos = get(dosyaxes,'CurrentPoint');
	y = pos(1,2);
	left = dfdata.wmin_ppm;
	right = dfdata.wmax_ppm;
	zlimits = get(dosyaxes, 'ZLim');
	zmax = zlimits(2);
	if(strcmp(button,'normal'))
		if(dfdata.nclicks)
			delete(dfdata.last_line);
			top = max(y, dfdata.last_y);
			bottom = min(y, dfdata.last_y);
			for r = 1:length(dfdata.Regions)
				rect = dfdata.Regions(r);
				rvert = get(rect,'Vertices');
				rtop = rvert(1,2);
				rbottom = rvert(3,2);
				if(top > rtop & bottom < rbottom)
					bottom = top;
					break;
				end
				if(top < rtop & top > rbottom)
					top = rbottom;
				end
				if(bottom < rtop & bottom > rbottom)
					bottom = rtop;
				end
			end
			if(top > bottom)
				rect = patch([left,right,right,left],[top,top,bottom,bottom],...
					[zmax,zmax,zmax,zmax], ...
					'Green', 'FaceAlpha',0.25, 'ButtonDownFcn',@delete_region);
				dfdata.Regions(end+1) = rect;
				% do stuff
			end
		else
			dfdata.last_y = y;
			dfdata.last_line = line([left,right],[y,y],[zmax,zmax], ...
				'Color', 'Black');
		end
		dfdata.nclicks = ~dfdata.nclicks;
	end	
	guidata(source,dfdata);
end

function delete_region(source,eventdata)
	dosyfigure = gcf();
	dosydata = guidata(dosyfigure);
	if(strcmp(get(dosyfigure,'SelectionType'),'normal'))
		dosyaxes = get(dosyfigure, 'CurrentAxes');
		click_callback(dosydata.window, eventdata);
	end
	if(strcmp(get(gcf,'SelectionType'),'alt') & ~dosydata.nclicks)
		dosydata.Regions(find(dosydata.Regions == source)) = [];
		guidata(source,dosydata);
		delete(source);
	end
end

function CallbackRender(source,eventdata)
	nspec = 5000;
	dosyfigure = gcf();
	dosydata = guidata(dosyfigure);
	regions = dosydata.Regions;
	windows = dosydata.windows;
	ak = -real(dosydata.ak);
	uk = dosydata.uk;
	dk = dosydata.dk;
	tau1 = dosydata.tau1;
	wmin = dosydata.wmin;
	wmax = dosydata.wmax;
	wstep = (wmax-wmin)/nspec;
	wp = dosydata.wp;
	sp = dosydata.sp;
	sfrq = dosydata.sfrq;
	spec = zeros(nspec,length(regions));
	bottom_top = zeros(length(regions),2);
	for r = 1:length(regions)
		vert = get(regions(r),'Vertices');
		bottom_top(r,:) = [vert(3,2),vert(1,2)];
		for window=1:size(windows,1)
			lwindow = windows(window,1);
			rwindow = windows(window,2);
			spec1min_win = floor(max((lwindow-wmin)*nspec / (wmax-wmin),0.0));
			spec1max_win = floor(min((rwindow-wmin)*nspec / (wmax-wmin),nspec+0.0))-1;
			selected = ak(:,window) > bottom_top(r,1) & ak(:,window) < bottom_top(r,2);
			for j=spec1min_win:spec1max_win
				w = wmin + j * wstep;
				weight = cos((w-lwindow)*2*pi/(rwindow-lwindow)+pi)*0.5 + 0.5;
				z = exp(-i*tau1*w);
				for k=1:size(uk,1)
					if(selected(k))
						if(1)
							spec(j+1,r) = spec(j+1,r) + tau1^2 * weight * dk(k,window) * ...
								(1/(1 - uk(k,window)/z)^2 - 0.5);
						else
							spec(j+1,r) = spec(j+1,r) + tau1 * weight * dk(k,window) * ...
								(1/(1 - uk(k,window)/z) - 0.5);
						end
					end
				end
			end
		end
	end
	specfig = figure();
	nplot = size(spec,2);
	pad = 0.1;
	axisfract = 0.2;
	plotheight = (1-pad)/(nplot+(nplot-1)*axisfract);
	for a=1:nplot
		ax(a) = axes('Position', [pad/2, pad/2 + (a-1)*plotheight*(1+axisfract), ...
			1-pad, plotheight]);
		plot(-(linspace(wmin, wmax, nspec)/sfrq - (-wp/2 - sp)), real(spec(:,a)));
	end
end

function zoomer(source,eventdata)
	dosyfigure = gcf();
	dosyaxes = get(dosyfigure, 'CurrentAxes');
	xlimits = get(dosyaxes, 'XLim');
	ylimits = get(dosyaxes, 'YLim');
	zlimits = get(dosyaxes, 'ZLim');
	zmax = zlimits(2);
	dfdata = guidata(dosyfigure);
	set(dfdata.window, 'Vertices', ...
		[[xlimits(1),xlimits(2),xlimits(2),xlimits(1)]',...
		[ylimits(2),ylimits(2),ylimits(1),xlimits(1)]', [zmax,zmax,zmax,zmax]'])
end

function [rotate]=fixer(source,eventdata)
	rotate = 1;
end


